#+ELEVENTY_BASE_DIR: ~/proj/static-blog/
#+ELEVENTY_COLLECTIONS: _posts
#+ELEVENTY_BASE_URL: https://sachachua.com
* TODO Org Mode: Merge top-level items in an item list  :org:
:PROPERTIES:
:CUSTOM_ID: org-mode-merge-top-level-items-in-an-item-list
:EXPORT_DATE: 2025-03-17T14:13:55-0400
:EXPORT_ELEVENTY_PERMALINK: /blog/2025/03/org-mode-merge-top-level-items-in-an-item-list/
:EXPORT_ELEVENTY_FILE_NAME: blog/2025/03/org-mode-merge-top-level-items-in-an-item-list/
:END:

I usually summarize Mastodon links, move them to
my Emacs News Org file, and then categorize them.
Today I accidentically categorized the links while
they were still in my Mastodon buffer, so I had
two lists with categories. I wanted to write some
Emacs Lisp to merge sublists based on the
top-level items. I could sort the list
alphabetically with ~C-c ^~ (org-sort) and then
delete the redundant top-level item lines, but
it's fun to tinker with Emacs Lisp.

Example input:

- Topic A:
  - Item 1
  - Item 2
    - Item 2.1
- Topic B:
  - Item 3
- Topic A:
  - Item 4
    - Item 4.1

Example output:

- Topic B:
  - Item 3
- Topic A:
  - Item 1
  - Item 2
    - Item 2.1
  - Item 4
    - Item 4.1

The sorting doesn't particularly matter to me, but I want the things under Topic A to be combined. Someday it might be nice to recursively merge other entries (ex: if there's another "Topic A: - Item 2" subitem like "Item 2.2"), but I don't need that yet.

Anyway, we can parse the list with
~org-list-to-lisp~ (which can even delete the
original list) and recreate it with
~org-list-to-org~, so then it's a matter of
transforming the data structure.

#+begin_src emacs-lisp
(defun my-org-merge-list-entries-at-point ()
  "Merge entries in a nested Org Mode list at point that have the same top-level item text."
  (interactive)
  (save-excursion
    (let* ((list-indentation (save-excursion
															 (goto-char (caar (org-list-struct)))
															 (current-indentation)))
					 (list-struct (org-list-to-lisp t))
           (merged-list (my-org-merge-list-entries list-struct)))
      (insert (org-ascii--indent-string (org-list-to-org merged-list) list-indentation)
							"\n"))))

(defun my-org-merge-list-entries (list-struct)
  "Merge an Org list based on its top-level headings"
	(cons (car list-struct)
				(mapcar
				 (lambda (g)
					 (list
						(car g)
						(let ((list-type (car (car (cdr (car (cdr g))))))
									(entries (seq-mapcat #'cdar (mapcar #'cdr (cdr g)))))
							(apply #'append (list list-type) entries nil))))
				 (seq-group-by #'car (cdr list-struct)))))
#+end_src

A couple of test cases:

#+begin_src emacs-lisp
(ert-deftest my-org-merge-list-entries ()
	(should
	 (equal
		(my-org-merge-list-entries
		 '(unordered ("Topic B:" (unordered ("Item 3")))))
		'(unordered ("Topic B:" (unordered ("Item 3"))))))
	(should
	 (equal
		(my-org-merge-list-entries
		 '(unordered ("Topic B:" (unordered ("Item 3")))
								 ("Topic A:"
									(unordered ("Item 1")
														 ("Item 2"
															(unordered ("Item 2.1")))))
								 ("Topic A:"
									(unordered
									 ("Item 4" (unordered ("Item 4.1")))))))
		'(unordered
			("Topic B:" (unordered ("Item 3")))
			("Topic A:"
			 (unordered ("Item 1")
									("Item 2" (unordered ("Item 2.1")))
									("Item 4" (unordered ("Item 4.1")))))))))
#+end_src

** Updating my custom links to also export to Org
:PROPERTIES:
:CUSTOM_ID: org-mode-merge-top-level-items-in-an-item-list-updating-my-custom-links-to-also-export-to-org
:END:

Because ~org-list-to-org~ uses the Org conversion
process, I need to make sure that my custom link
functions also export to Org as a format. For
example, in Emacs News, I use a [[dotemacs:package-links][package:]] link to
make it easy to link to packages in both Emacs and
in exported HTML. When I first ran my code, the
links got replaced with their URLs, which isn't
what I wanted. Turned out that I needed to add a
case handling exporting to =org= format, like
this:

#+begin_src emacs-lisp :tangle no
(defun my-org-package-export (link description format &optional arg)
  (let* ((package-info (car (assoc-default (intern link) package-archive-contents)))
         (package-source (and package-info (package-desc-archive package-info)))
         (path (format
                (cond
								 ((null package-source) link)
                 ((string= package-source "gnu") "https://elpa.gnu.org/packages/%s.html")
                 ((string= package-source "melpa") "https://melpa.org/#/%s")
                 ((string= package-source "nongnu") "https://elpa.nongnu.org/nongnu/%s.html")
                 (t (error 'unknown-source)))
                link))
         (desc (or description link)))
		(if package-source
				(cond
				 ((eq format '11ty) (format "<a target=\"_blank\" href=\"%s\">%s</a>" path desc))
				 ((eq format 'html) (format "<a target=\"_blank\" href=\"%s\">%s</a>" path desc))
				 ((eq format 'wp) (format "<a target=\"_blank\" href=\"%s\">%s</a>" path desc))
				 ((eq format 'latex) (format "\\href{%s}{%s}" path desc))
				 ((eq format 'texinfo) (format "@uref{%s,%s}" path desc))
				 ((eq format 'ascii) (format "%s <%s>" desc path))
				 ((eq format 'org) (org-link-make-string (concat "package:" link) description)) ;; added this line
				 (t path))
			desc)))
#+end_src

